Unit OpenGL_Standards;

Interface

Uses dglOpenGL;

Type

  TVector2f = Record
    x: GLfloat;
    y: GLfloat;
  End;

  TVector3f = Record
    x: GLfloat;
    y: GLfloat;
    z: GLfloat;
  End;

  TVector4f = Record
    x: GLfloat;
    y: GLfloat;
    z: GLfloat;
    w: GLfloat;
  End;

  TVector2i = Record
    x: GLint;
    y: GLint;
  End;

  TVector3i = Record
    x: GLint;
    y: GLint;
    z: GLint;
  End;

  TVector4i = Record
    x: GLint;
    y: GLint;
    z: GLint;
    w: GLint;
  End;

  TVector2b = Record
    x: GLubyte;
    y: GLubyte;
  End;

  TVector3b = Record
    x: GLubyte;
    y: GLubyte;
    z: GLubyte;
  End;

  TVector4b = Record
    x: GLubyte;
    y: GLubyte;
    z: GLubyte;
    w: GLubyte;
  End;

  TOpenGL_0_to_15_Matrix = Array[0..15] Of GLfloat;
  TOpenGL3x3Matrix = Array[0..2, 0..2] Of GLfloat;
  TOpenGL4x4Matrix = Array[0..3, 0..3] Of GLfloat;
  TVector2fDynArr = Array Of TVector2f;
  TVector3fDynArr = Array Of TVector3f;

Const
  ZeroV2f: TVector3f = (x: 0; y: 0);
  ZeroV3f: TVector3f = (x: 0; y: 0; z: 0);
  ZeroV4f: TVector4f = (x: 0; y: 0; z: 0; W: 0);
  ZeroV2i: TVector3i = (x: 0; y: 0);
  ZeroV3i: TVector3i = (x: 0; y: 0; z: 0);
  ZeroV4i: TVector4i = (x: 0; y: 0; z: 0; W: 0);
  ZeroOpenGL_0_to_15_Matrix: TOpenGL_0_to_15_Matrix = (0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
  IdentityOpenGL_0_to_15_Matrix: TOpenGL_0_to_15_Matrix = (1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1);

  // Conversion Functions
Function V2(x, y: GLfloat): TVector2f;
Function V3(x, y, z: GLfloat): TVector3f;
Function V3b(x, y, z: GLubyte): TVector3b;
Function V3i(x, y, z: GLint): TVector3i;
Function V4(x, y, z, w: GLfloat): TVector4f;
Function V4b(x, y, z, w: GLuByte): TVector4b;
// Mathematic Functions
Function LenV2(v: Tvector2f): GLfloat;
Function LenV3(v: Tvector3f): GLfloat;
Function LenV4(v: Tvector4f): GLfloat;
Function AddV2(v1, v2: TVector2f): Tvector2f;
Function SubV2(v1, v2: TVector2f): Tvector2f;
Function AddV3(v1, v2: TVector3f): Tvector3f;
Function SubV3(v1, v2: TVector3f): Tvector3f;
Function AddV4(v1, v2: TVector4f): Tvector4f;
Function SubV4(v1, v2: TVector4f): Tvector4f;
Function ScaleV2(Scalar: GLfloat; v: Tvector2f): Tvector2f;
Function ScaleV3(Scalar: GLfloat; v: Tvector3f): Tvector3f;
Function ScaleV4(Scalar: GLfloat; v: Tvector4f): Tvector4f;
Function ScaleV4b(Scalar: GLfloat; v: Tvector4b): Tvector4b;
Function CrossProdV3(v1, v2: Tvector3f): Tvector3f;
Function NormalizeV3(Value: Tvector3f): Tvector3f;
Function DotProdV3(v1, v2: TVector3f): GLfloat;
// Special OpenGL Functions
Function GetNormal(EbenenPunkt1, EbenenPunkt2, EbenenPunkt3, Punktinnen: Tvector3f): Tvector3f;
Procedure SetNormal(Var Normal: TVector3f); // Setzt Die Normale, Var nur deswegen weil damit nur der Pointer bergeben wird => schneller
Procedure GenerateShadowMatrix(Var ShadowMatrix: TOpenGL_0_to_15_Matrix; Const normal, point: TVector3f; lightX, lightY, lightZ, lightW: GLfloat); overload;
Procedure GenerateShadowMatrix(Var ShadowMatrix: TOpenGL_0_to_15_Matrix; Const normal, point: TVector3f; light: Tvector4f); overload;

Var
  aCosinus, aSinus: Array[0..359] Of GLFloat;

Implementation

Function V2(x, y: GLfloat): TVector2f;
Begin
  result.x := x;
  result.y := y;
End;

Function V3(x, y, z: GLfloat): TVector3f;
Begin
  result.x := x;
  result.y := y;
  result.z := z;
End;

Function V3i(x, y, z: GLint): TVector3i;
Begin
  result.x := x;
  result.y := y;
  result.z := z;
End;

Function V3b(x, y, z: GLubyte): TVector3b;
Begin
  result.x := x;
  result.y := y;
  result.z := z;
End;

Function V4(x, y, z, w: GLfloat): TVector4f;
Begin
  result.x := x;
  result.y := y;
  result.z := z;
  result.w := w;
End;

Function V4b(x, y, z, w: GLuByte): TVector4b;
Begin
  result.x := x;
  result.y := y;
  result.z := z;
  result.w := w;
End;

Function lenv2(v: Tvector2f): GLfloat;
Begin
  result := sqrt(sqr(v.x) + sqr(v.y));
End;

Function lenv3(v: Tvector3f): GLfloat;
Begin
  result := sqrt(sqr(v.x) + sqr(v.y) + sqr(v.z));
End;

Function lenv4(v: Tvector4f): GLfloat;
Begin
  result := sqrt(sqr(v.x) + sqr(v.y) + sqr(v.z) + sqr(v.w));
End;

Function AddV2(v1, v2: TVector2f): Tvector2f;
Begin
  result.x := v1.x + v2.x;
  result.y := v1.y + v2.y;
End;

Function SubV2(v1, v2: TVector2f): Tvector2f;
Begin
  result.x := v1.x - v2.x;
  result.y := v1.y - v2.y;
End;

Function AddV3(v1, v2: TVector3f): Tvector3f;
Begin
  result.x := v1.x + v2.x;
  result.y := v1.y + v2.y;
  result.z := v1.z + v2.z;
End;

Function SubV3(v1, v2: TVector3f): Tvector3f;
Begin
  result.x := v1.x - v2.x;
  result.y := v1.y - v2.y;
  result.z := v1.z - v2.z;
End;

Function AddV4(v1, v2: TVector4f): Tvector4f;
Begin
  result.x := v1.x + v2.x;
  result.y := v1.y + v2.y;
  result.z := v1.z + v2.z;
  result.w := v1.w + v2.w;
End;

Function SubV4(v1, v2: TVector4f): Tvector4f;
Begin
  result.x := v1.x - v2.x;
  result.y := v1.y - v2.y;
  result.z := v1.z - v2.z;
  result.w := v1.z - v2.w;
End;

Function ScaleV2(Scalar: GLfloat; v: Tvector2f): Tvector2f;
Begin
  result.x := scalar * v.x;
  result.y := scalar * v.y;
End;

Function ScaleV3(Scalar: GLfloat; v: Tvector3f): Tvector3f;
Begin
  result.x := scalar * v.x;
  result.y := scalar * v.y;
  result.z := scalar * v.z;
End;

Function ScaleV4(Scalar: GLfloat; v: Tvector4f): Tvector4f;
Begin
  result.x := scalar * v.x;
  result.y := scalar * v.y;
  result.z := scalar * v.z;
  result.w := scalar * v.w;
End;

Function ScaleV4b(Scalar: GLfloat; v: Tvector4b): Tvector4b;
Begin
  result.x := round(scalar * v.x);
  result.y := round(scalar * v.y);
  result.z := round(scalar * v.z);
  result.z := round(scalar * v.z);
End;

Function CrossProdV3(v1, v2: Tvector3f): Tvector3f;
Begin
  result := v3(
    v1.y * v2.z - v1.z * v2.y,
    v1.z * v2.x - v1.x * v2.z,
    v1.x * v2.y - v1.y * v2.x);
End;

Function NormalizeV3(Value: Tvector3f): Tvector3f;
Var
  betrag: GLfloat;
Begin
  betrag := lenv3(value);
  If Betrag <> 0 Then
    result := ScaleV3(1 / Betrag, Value)
  Else
    result := v3(0.0, 0.0, 0.0);
End;

Function DotProdV3(v1, v2: TVector3f): GLfloat;
Begin
  result := v1.x * v2.x + v1.y * v2.y + v1.z * v2.z;
End;

Function GetNormal(EbenenPunkt1, EbenenPunkt2, EbenenPunkt3, Punktinnen: Tvector3f): Tvector3f;
Var
  erg, d1, d2: Tvector3f;
Begin
  d1 := subv3(EbenenPunkt1, EbenenPunkt2); // Berechnen der Richtungsvektoren der Ebene
  d2 := subv3(EbenenPunkt1, EbenenPunkt3); // Berechnen der Richtungsvektoren der Ebene
  erg := CrossprodV3(d1, d2); // Berechnen eines Senkrecht auf der Ebene Stehenden Vektors
  erg := normalizeV3(erg); // Normieren des Vektors
  // Schaun ob unser Normalenvektor auf den Halbraum zeigt in dem unser Punktinnen liegt
  d1 := subv3(EbenenPunkt1, Punktinnen);
  If DotProdV3(erg, d1) < 0 Then // Wenn nicht dann mus er umgedreht werden
    erg := v3(-erg.x, -erg.y, -erg.z);
  result := erg;
End;

Procedure GenerateShadowMatrix(Var ShadowMatrix: TOpenGL_0_to_15_Matrix; Const normal, point: TVector3f; light: Tvector4f);
Begin
  GenerateShadowMatrix(ShadowMatrix, normal, point, light.x, light.y, light.z, light.w);
End;

Procedure GenerateShadowMatrix(Var ShadowMatrix: TOpenGL_0_to_15_Matrix; Const normal, point: TVector3f; lightX, lightY, lightZ, lightW: GLfloat);
Var
  d, dot: Real;
Begin
  d := -normal.X * point.X - normal.Y * point.Y - normal.Z * point.Z;
  dot := normal.X * lightX + normal.Y * lightY + normal.Z * lightZ + d * lightW;
  ShadowMatrix[0] := -lightX * normal.X + dot;
  ShadowMatrix[1] := -lightY * normal.X;
  ShadowMatrix[2] := -lightZ * normal.X;
  ShadowMatrix[3] := -lightW * normal.X;
  ShadowMatrix[4] := -lightX * normal.Y;
  ShadowMatrix[5] := -lightY * normal.Y + dot;
  ShadowMatrix[6] := -lightZ * normal.Y;
  ShadowMatrix[7] := -lightW * normal.Y;
  ShadowMatrix[8] := -lightX * normal.Z;
  ShadowMatrix[9] := -lightY * normal.Z;
  ShadowMatrix[10] := -lightZ * normal.Z + dot;
  ShadowMatrix[11] := -lightW * normal.Z;
  ShadowMatrix[12] := -lightX * d;
  ShadowMatrix[13] := -lightY * d;
  ShadowMatrix[14] := -lightZ * d;
  ShadowMatrix[15] := -lightW * d + dot;
End;

Procedure SetNormal(Var Normal: TVector3f);
Begin
  //  If glIsEnabled(gl_lighting) Then // Man Braucht die Normalen manchmal auch ohne Lighting
  glnormal3fv(@Normal);
End;

Procedure SetSinCos;
Var
  x: integer;
Begin
  For x := 0 To 359 Do Begin
    asinus[x] := sin(x * Pi / 180);
    aCOsinus[x] := cos(x * Pi / 180);
  End;
End;

Initialization
  SetSinCos;

End.

