Unit newtonprimitives;

{$mode Delphi}

Interface

Uses
  sysutils, // Debugg
  maths3d,
  dglOpenGL,
  OpenGL_Standards,
  newtonimport;

Const
  Schwerkraft = 9.82;

Type
  TNewtonConCaveHull = Class
  private
    fNewtonBody: Pnewtonbody;
    Fnewtonworld: Pnewtonworld;

    Fpoints: Array Of TVector3fDynArr;
    fMatrix: TOpenGL4x4Matrix; // Used to retrieve the matrix from newton
    fPosition: Tvector3f;
    Procedure Setmass(Value: Integer);
  public
    Property Mass: Integer write setmass;
    Property Newtonbody: Pnewtonbody read fNewtonBody;
    Constructor Create(Position: TVector3f);
    Destructor destroy; override;
    Procedure SetPolyPoints(value: Array Of TVector3fDynArr); // Für jedes Convexe Element brauchen wir ein Eigenes Array ;)
    Procedure AddtoNewtonworld(Const Newtonworld: Pnewtonworld; Mass_: integer; Offset: Tvector3f; Material: integer);
    Procedure Render;
    //    Procedure setBarierGroup(ID: Gluint; Const Newtonworld: Pnewtonworld);
  End;

  TNewtonConvexHull = Class // Wird aber nur für Feste Objecte benutzt, oder ?
  private
    fNewtonBody: Pnewtonbody;
    Fpoints: Array Of TVector3f;
    fMatrix: TOpenGL4x4Matrix; // Used to retrieve the matrix from newton
    fPosition: Tvector3f;
  public
    Property Body: Pnewtonbody read fNewtonBody;
    Constructor Create(Position: TVector3f);
    Destructor destroy; override;
    Procedure SetPolyPoints(value: Array Of Tvector3f);
    Procedure AddtoNewtonworld(Const Newtonworld: Pnewtonworld; Mass, Material: Integer);
    Procedure Render;
  End;

Function CreateMaterialGroup(Const Newtonworld: Pnewtonworld): integer;
Procedure SetGroupcollision(Const newtonworld: Pnewtonworld; PLayerid, Fixworldid: Integer);
Procedure ForceAndTorqueCallback( const body : PNewtonBody; timestep : Float; threadIndex : int ); cdecl;

Implementation

Function CreateMaterialGroup(Const Newtonworld: Pnewtonworld): integer;
Begin
  result := NewtonMaterialCreateGroupID(newtonworld);
End;

Procedure SetGroupcollision(Const newtonworld: Pnewtonworld; PLayerid, Fixworldid: Integer);
Begin
  NewtonMaterialSetDefaultCollidable(newtonworld, Playerid, FixWorldid, 1);
End;

Procedure ForceAndTorqueCallback( const body : PNewtonBody; timestep : Float; threadIndex : int ); cdecl;
Var
  Mass: Single;
  Inertia: TVector3f;
  Force: TVector3f;
Begin
  NewtonBodyGetMassMatrix(Body, @Mass, @Inertia.x, @Inertia.y, @Inertia.z);
  //  Force := V3(0, -9.8 * Mass, 0); // Echte Erdbeschleunigung.
  Force := V3(0, -Schwerkraft * Mass, 0);
  NewtonBodyAddForce(Body, @Force.x);
End;

Constructor TNewtonConvexHull.Create(Position: TVector3f);
Begin
  Inherited create;
  setlength(fpoints, 0);
  fPosition.x := position.x;
  fPosition.y := position.y;
  fPosition.z := position.z;
  fNewtonBody := Nil;
End;

Destructor TNewtonConvexHull.destroy;
Begin
  setlength(fpoints, 0);
  //  Inherited Destroy; // Braucht man net da wir von Tobject abgeleitet haben.
End;

Procedure TNewtonConvexHull.SetPolyPoints(value: Array Of Tvector3f);
Var
  x: integer;
Begin
  // Übernehmen der Punkte
  setlength(fpoints, high(value) + 1);
  For x := 0 To high(Value) Do Begin
    Fpoints[x].x := value[x].x;
    Fpoints[x].y := value[x].y;
    Fpoints[x].z := value[x].z;
  End;
End;

Procedure TNewtonConvexHull.AddtoNewtonworld(Const Newtonworld: Pnewtonworld; Mass, Material: Integer);
Var
  tp: Array  of TVector3f;
  Collision: PNewtonCollision;
  i: Integer;
  Inertia, Origin: TVector3f;
begin
  if high(fpoints) = -1 then exit;

  //setlength(tp, high(fpoints) + 1);
  //for i := 0 to high(tp) do
  //  tp[i] := SubV3(Fpoints[i],Fposition);

  Collision := NewtonCreateBox(NewtonWorld, 1,1,1  ,nil);
     //NewtonCreateConvexHull(newtonworld, high(fpoints)+1, @tp[0].x, 12, 0,  Nil);
  // Create the rigid body
  fNewtonBody := NewtonCreateBody(NewtonWorld, Collision);
  // Da wir unbewegliche Objecte Haben ist das Berechnen des Schwerpunktes sehr einfach !!!
  if mass = 0 then
    NewtonBodySetMassMatrix(fNewtonBody, 0, 0, 0, 0)
  else begin
     { -- Klammert man diese Zeilen aus, dann gibts keine AV,
    Origin := Fpoints[0];
    for i := 1 to High(Fpoints) do
      Origin := Addv3(Origin, Fpoints[i]);
    Origin := ScaleV3(1 / (high(Fpoints) +1 ), Origin);
    //    -- }
    NewtonConvexCollisionCalculateInertialMatrix(Collision, @Inertia, @Origin);
    NewtonBodySetMassMatrix(fNewtonBody, mass, inertia.x * mass, inertia.y * mass, inertia.z * mass)
  end;
  // Remove the collider, we don't need it anymore
  NewtonReleaseCollision(NewtonWorld, Collision);
  // Now set the position of the body's matrix
  NewtonBodyGetMatrix(fNewtonBody, @FMatrix[0, 0]);
  fMatrix[3, 0] := fPosition.x;
  fMatrix[3, 1] := fPosition.y;
  fMatrix[3, 2] := fPosition.z;
  NewtonBodySetMatrix(fNewtonBody, @fMatrix[0, 0]);
  //   Finally set the callback in which the forces on this body will be applied
  NewtonBodySetForceAndTorqueCallBack(fNewtonBody, ForceAndTorqueCallBack);
  NewtonBodySetMaterialGroupID(fnewtonbody, Material);
End;

Procedure TNewtonConvexHull.render;
Var
  x: Integer;
  b: Boolean;
  m: TOpenGL_0_to_15_Matrix;
Begin
  b := glIsEnabled(gl_lighting);
  If b Then
    gldisable(gl_lighting);
  glPushMatrix;
  NewtonBodyGetMatrix(fnewtonbody,@m[0]);
  glMultMatrixf(@m[0]);
  //glTranslatef(fposition.x, fposition.y, fposition.z);
  glBindTexture(GL_TEXTURE_2D, 0);
  glcolor3f(1, 1, 1);
  glbegin(GL_LINE_LOOP);
  For x := 0 To High(Fpoints) Do
    glvertex3f(fpoints[x].x, fpoints[x].y, fpoints[x].z);
  glend;
  glPopMatrix;
  If b Then
    glenable(gl_lighting);
End;

Constructor TNewtonConCaveHull.Create(Position: TVector3f);
Begin
  Inherited create;
  setlength(fpoints, 0, 0);
  fPosition.x := position.x;
  fPosition.y := position.y;
  fPosition.z := position.z;
  FNewtonWorld := Nil;
End;

Destructor TNewtonConCaveHull.destroy;
Var
  x: integer;
Begin
  For x := 0 To High(fpoints) Do
    setlength(fpoints[x], 0);
  setlength(fpoints, 0);
  //  Inherited Destroy; // Braucht man net da wir von Tobject abgeleitet haben.
End;

Procedure TNewtonConCaveHull.Setmass(Value: Integer);
Begin
  NewtonBodySetMassMatrix(fNewtonBody, Value, 0, 0, 0);
(*
Removed vor version 2.0
*)
//  If (Value <> 0) And assigned(FNewtonWorld) Then
//    NewtonWorldUnfreezeBody(FNewtonWorld, fNewtonBody);
End;

Procedure TNewtonConCaveHull.SetPolyPoints(value: Array Of TVector3fDynArr);
Var
  y, x: integer;
Begin
  // Übernehmen der Punkte
  setlength(fpoints, high(value) + 1);
  For x := 0 To high(Value) Do Begin
    setlength(fpoints[x], high(value[x]) + 1);
    For y := 0 To High(Value[x]) Do Begin
      Fpoints[x][y].x := value[x][y].x;
      Fpoints[x][y].y := value[x][y].y;
      Fpoints[x][y].z := value[x][y].z;
    End;
  End;
  //  Finertia := Getinertia;
End;

Procedure TNewtonConCaveHull.AddtoNewtonworld(Const Newtonworld: Pnewtonworld; Mass_: integer; Offset: Tvector3f; Material: integer);
Var
  Collision: Array Of PNewtonCollision;
  rescollider: PNewtonCollision;
  x: integer;
  inertia, v2: TVector3f;
Begin
  If High(Fpoints) = -1 Then exit;
  FNewtonWorld := Newtonworld;
  // Erstellen der Einzelnen Collides
  Setlength(Collision, High(Fpoints) + 1);
  For x := 0 To High(Fpoints) Do Begin
    Collision[x] := NewtonCreateConvexHull(newtonworld, high(Fpoints[x]) + 1, @fpoints[x][0], 12,0, Nil);
  End;
  rescollider := newtoncreateCompoundCollision(newtonworld, High(Collision) + 1, @Collision[0]);

  fNewtonBody := NewtonCreateBody(NewtonWorld, rescollider);
  // Remove the collider, we don't need it anymore

  NewtonConvexCollisionCalculateInertialMatrix(rescollider, @inertia, @v2);
  //  If Not Gleich(v3(0, 0, 0), offset) Then
  //    inertia := v3(offset.x * mass, offset.y * mass, offset.z * mass);
    //    inertia := vektoradd(inertia, offset);

  Setlength(Collision, 0);
  //  Da wir Bewegliche Objecte Haben wird es nun aufwendiger !!

//  NewtonBodySetMassMatrix(fNewtonBody, Mass, finertia.x * mass, finertia.y * mass, finertia.z * mass);
  NewtonBodySetMassMatrix(fNewtonBody, Mass_, inertia.x * mass_, inertia.y * mass_, inertia.z * mass_);

  //  // Now set the position of the body's matrix
  NewtonBodyGetMatrix(fNewtonBody, @FMatrix[0, 0]);
  fMatrix[3, 0] := fPosition.x;
  fMatrix[3, 1] := fPosition.y;
  fMatrix[3, 2] := fPosition.z;
  NewtonBodySetMatrix(fNewtonBody, @fMatrix[0, 0]);
  v2 := AddV3(v2, offset);
  //  v2.y := v2.y - 1; // Wir setzen den Schwerpunkt ein klein wenig nach unten , dann liegen die Objekte besser.

  NewtonBodySetCentreOfMass(fNewtonBody, @v2);

  NewtonReleaseCollision(NewtonWorld, rescollider);

  //   Finally set the callback in which the forces on this body will be applied
  NewtonBodySetForceAndTorqueCallBack(fNewtonBody, ForceAndTorqueCallBack);

  //  FGroupid := CreateMaterialGroup(newtonworld);

  NewtonBodySetMaterialGroupID(fnewtonbody, Material);

  // Dafür sorgen das unser Object Beweglich bleibt
(*
Removed für ver 2.0
*)
//  NewtonWorldUnfreezeBody(NewtonWorld, fNewtonBody);
//    NewtonBodySetAutoFreeze(fNewtonBody, 0);  // Diese Zeile Wäre Tödlich
End;
{
Procedure TNewtonConCaveHull.setBarierGroup(ID: Gluint; Const Newtonworld: Pnewtonworld);
Begin
  //  NewtonMaterialSetDefaultFriction(NewtonWorld, FGroupid, id, 0.0, 0.0);
//  NewtonMaterialSetDefaultFriction(NewtonWorld, FGroupid, id, 1.1, 1.0);
End;
}

Procedure TNewtonConCaveHull.render;
Var
  y, x: Integer;
Begin
  If Not Assigned(FNewtonBody) Then exit;
  glPushMatrix;
  // Hohlen der Positionsdaten des Körpers
  NewtonBodyGetMatrix(FNewtonBody, @fMatrix[0, 0]);
  glBindTexture(GL_TEXTURE_2D, 0);
  glcolor3f(1, 1, 1);
  // glTranslatef(fposition.x, fposition.y, fposition.z);
  glMultMatrixf(@fMatrix[0, 0]);
  glBindTexture(GL_TEXTURE_2D, 0);
  glcolor3f(1, 1, 0);
  For x := 0 To High(Fpoints) Do Begin
    glbegin(GL_LINE_LOOP);
    For y := 0 To High(Fpoints[x]) Do
      glvertex3f(fpoints[x][y].x, fpoints[x][y].y, fpoints[x][y].z);
    glend;
  End;
  glcolor3f(1, 1, 1);
  //  glcolor3f(0, 1, 0);
  //  glbegin(GL_POINTS);
  //  glvertex3f(finertia.x * 70, finertia.y * 70, finertia.z * 70);
  //  glvertex3f(finertia.x * 70 + 1, finertia.y * 70, finertia.z * 70);
  //  glvertex3f(finertia.x * 70 - 1, finertia.y * 70, finertia.z * 70);
  //  glvertex3f(finertia.x * 70, finertia.y * 70, finertia.z * 70 + 1);
  //  glvertex3f(finertia.x * 70, finertia.y * 70, finertia.z * 70 - 1);
  //  glend;
  glPopMatrix;
End;

End.

